/*

i8k.dll - a plugin for serious samurize, interfaces with fanio.sys system monitoring driver

*/

#include <windows.h>
#include <stdio.h>
#include "samurize.h"
#include "fanioctl.h"

#define NODRIVER "ERROR: Could not load fanio driver."
#define NOVALUE "ERROR: No value was present."
#define RETURNLENGTH 512

#define FAN1ID 0
#define FAN2ID 1

const UINT64 oneSecond = 10000000;
void GetSysInfo();
void logInfoString(char* s, char* dest);

HWND hSamurize;
HANDLE glb_hHndFile;
int isDriverLoaded;
int swapFans;
int is8500;

int CPUTEMPID;
int GPUTEMPID;
int DIMMTEMPID;
int OTHERTEMPID;

char *sysInfo;
char *prodName;

struct i8k_meter{
	UINT64 lastCall;
	char* returnVal;
	UINT deviceVal;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
////////////////////////////  DLL Functions  ////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


void startup(HANDLE hModule)
{
	sysInfo = (char *)malloc(16384);
	prodName = (char *)malloc(128);
	// access the driver
	glb_hHndFile = CreateFile( "\\\\.\\FanIODev",
								GENERIC_WRITE,
								FILE_SHARE_WRITE,
								NULL,
								OPEN_EXISTING,
								0,
								NULL );

	// check to see if driver loaded properly
	if( glb_hHndFile == INVALID_HANDLE_VALUE ) isDriverLoaded = 0;
	else
	{
		isDriverLoaded = 1;
		CPUTEMPID = 0;
		GPUTEMPID = 1;
		DIMMTEMPID = 2;
		OTHERTEMPID = 3;
		swapFans = 0;
		is8500 = 0;

		GetSysInfo();
		logInfoString( "Product name:", prodName );
		
		if( (strstr(prodName,"Inspiron") && strstr(prodName,"8500"))
		||  (strstr(prodName,"Inspiron") && strstr(prodName,"8600"))
		||  (strstr(prodName,"Latitude") && strstr(prodName,"D800"))
		||  (strstr(prodName,"Latitude") && strstr(prodName,"D500"))
		||  (strstr(prodName,"Latitude") && strstr(prodName,"D400"))
		||  (strstr(prodName,"Precision") && strstr(prodName,"M60")) 
		||  (strstr(prodName,"Latitude") && strstr(prodName,"D600")) )
		{
			swapFans = 1;

			if( !strstr(prodName,"D400") && !strstr(prodName,"D500") && !strstr(prodName,"D600") )
			{
				is8500 = 1;
				GPUTEMPID = 2;
				DIMMTEMPID = 1;
			}
	    }

	}
}

void cleanup()
{
	free(sysInfo);
	free(prodName);
	if(isDriverLoaded) CloseHandle(glb_hHndFile);
	isDriverLoaded = 0;
}

BOOL APIENTRY DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
{
	
	switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			startup(hModule);
			break;
		case DLL_PROCESS_DETACH:
			cleanup();
			break;
	}
    return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////  Samurize Functions  //////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


SAMURIZE_PLUGIN_CHAR init()
{	
	return "GetCPUTemp|GetGPUTemp|GetDIMMTemp|GetOTHERTemp|GetFAN1RPM|GetFAN2RPM|GetFAN1Speed|GetFAN2Speed";
}

SAMURIZE_PLUGIN_CHAR getparam(char* func)
{
	if(strstr(func, "Temp")) return "refresh rate (seconds)=5|temperature unit (c or f)=c|show degree symbol=no|";
	return "refresh rate(seconds)=5|";
}


SAMURIZE_PLUGIN_INT dllstartup(HWND hwnd, int dlltype)
{
	hSamurize = hwnd;
	i8k_meter *defaultMeter;

	defaultMeter = (i8k_meter *)malloc(sizeof(i8k_meter));
	defaultMeter->returnVal = (char *)malloc(RETURNLENGTH);
	defaultMeter->lastCall = 0;

	return (int)defaultMeter;
}

SAMURIZE_PLUGIN_INT dllshutdown(int id)
{
	i8k_meter *defaultMeter;

	defaultMeter = (i8k_meter *) id;
	free(defaultMeter->returnVal);
	free(defaultMeter);

	return 0;
}

SAMURIZE_PLUGIN_INT dlltype()
{
	// 1 is source plugin
	// 2 is vis
	// 3 is source and vis
	return 1;
}

SAMURIZE_PLUGIN_CHAR getinfo(int infonum)
{
	switch (infonum) {
        case 0: return "i8k fanio interface";
        case 1: return "torsius";
        case 2: return "1.0";
        case 3: return "050126";
        case 4: return "050131";
        case 5: return "www.samurize.com";
		case 6: return "torsius@gmail.com";
	}
	return "";
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
//////////////////////////  Internal Functions  /////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


int NeedsRefresh(i8k_meter *meter_data, int updateInterval)
{
	UINT64 now;

	GetSystemTimeAsFileTime((LPFILETIME)&now);
	if (now >= meter_data->lastCall + updateInterval*oneSecond)
	{
		meter_data->lastCall = now;
		return 1;
	}

	return 0;
}

int GetTemp(i8k_meter *meter_data, int which)
{
	int result;
	ULONG ReturnedLength;

	result = DeviceIoControl(
					glb_hHndFile,
					IOCTL_FANIO_GET_TEMP,
					&which,
					sizeof(int),
					&(meter_data->deviceVal),
					sizeof(UINT),
					&ReturnedLength,
					NULL );

	if( meter_data->deviceVal == -1 ) return 0;
	return 1;
}

int GetFanRPM(i8k_meter *meter_data, int which)
{
	int result;
	ULONG ReturnedLength;

	result = DeviceIoControl(
					glb_hHndFile,
					IOCTL_FANIO_GET_FAN_RPM,
					&which,
					sizeof(int),
					&(meter_data->deviceVal),
					sizeof(UINT),
					&ReturnedLength,
					NULL );

	if( meter_data->deviceVal == -1 ) return 0;
	if(is8500) meter_data->deviceVal /= 30;
	return 1;
}

int GetFanSpeed(i8k_meter *meter_data, int which)
{
	int result;
	ULONG ReturnedLength;

	result = DeviceIoControl(
					glb_hHndFile,
					IOCTL_FANIO_GET_FAN_SPEED,
					&which,
					sizeof(int),
					&(meter_data->deviceVal),
					sizeof(UINT),
					&ReturnedLength,
					NULL );

	if( meter_data->deviceVal == -1 ) return 0;
	return 1;
}

void GetSysInfo()
{
	int result;
	ULONG ReturnedLength;
    
	DeviceIoControl(
					glb_hHndFile,
					IOCTL_FANIO_GET_SYSTEM_INFO,
					NULL,
					0,
					sysInfo,
					16384,
					&ReturnedLength,
					NULL );
}

void logInfoString( char* s, char* dest )
{
  char* u = strstr( sysInfo, s );
  if( !u )
    return;

  char *i = u;
  char c;
  
  while( *i && *i!='\n' && *i!='\r' )
    i++;

  i--;

  for( ;; )
  {
    if( *i && isspace(*i) ) 
      i--;
    else
      break;
  }

  i++;

  if( *i )
    c = *i;

  *i = '\0';
//  log( "--> %s <--", u );

  if( dest != NULL )
  {
    char* m = u+strlen(s);
    while( isspace(*m) ) m++;
    strcpy( dest, m );
  }

  *i = c;
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
////////////////////////////  User Functions  ///////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

SAMURIZE_PLUGIN_CHAR GetCPUTemp(int id, char *updateIntervalString, char *tempUnit, char *degree)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetTemp(meter_data, CPUTEMPID)) strcpy(meter_data->returnVal, NOVALUE);
		else
		{
			// see if needs unit conversion
			if(tempUnit[0]=='f') meter_data->deviceVal = (meter_data->deviceVal*9)/5 + 32;
			if(degree[0]=='y') sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
			else sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
		}
	}

	return meter_data->returnVal;
}

SAMURIZE_PLUGIN_CHAR GetGPUTemp(int id, char *updateIntervalString, char *tempUnit, char *degree)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetTemp(meter_data, GPUTEMPID)) strcpy(meter_data->returnVal, NOVALUE);
		else
		{
			// see if needs unit conversion
			if(tempUnit[0]=='f') meter_data->deviceVal = (meter_data->deviceVal*9)/5 + 32;
			if(degree[0]=='y') sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
			else sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
		}
	}

	return meter_data->returnVal;
}

SAMURIZE_PLUGIN_CHAR GetDIMMTemp(int id, char *updateIntervalString, char *tempUnit, char *degree)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetTemp(meter_data, DIMMTEMPID)) strcpy(meter_data->returnVal, NOVALUE);
		else
		{
			// see if needs unit conversion
			if(tempUnit[0]=='f') meter_data->deviceVal = (meter_data->deviceVal*9)/5 + 32;
			if(degree[0]=='y') sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
			else sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
		}
	}

	return meter_data->returnVal;
}

SAMURIZE_PLUGIN_CHAR GetOTHERTemp(int id, char *updateIntervalString, char *tempUnit, char *degree)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetTemp(meter_data, OTHERTEMPID)) strcpy(meter_data->returnVal, NOVALUE);
		else
		{
			// see if needs unit conversion
			if(tempUnit[0]=='f') meter_data->deviceVal = (meter_data->deviceVal*9)/5 + 32;
			if(degree[0]=='y') sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
			else sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
		}
	}

	return meter_data->returnVal;
}

SAMURIZE_PLUGIN_CHAR GetFAN1RPM(int id, char *updateIntervalString)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetFanRPM(meter_data, FAN1ID)) strcpy(meter_data->returnVal, NOVALUE);
		else sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
	}

	return meter_data->returnVal;
}

SAMURIZE_PLUGIN_CHAR GetFAN2RPM(int id, char *updateIntervalString)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetFanRPM(meter_data, FAN2ID)) strcpy(meter_data->returnVal, NOVALUE);
		else sprintf(meter_data->returnVal, "%d", meter_data->deviceVal);
	}

	return meter_data->returnVal;
}

SAMURIZE_PLUGIN_CHAR GetFAN1Speed(int id, char *updateIntervalString)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetFanSpeed(meter_data, FAN1ID)) strcpy(meter_data->returnVal, NOVALUE);
		else 
		switch(meter_data->deviceVal)
		{
			case 0:
				strcpy( meter_data->returnVal, "Off");
				break;
			case 1:
				strcpy( meter_data->returnVal, "Slow");
				break;
			case 2:
				strcpy( meter_data->returnVal, "High");
				break;
			default:
				strcpy( meter_data->returnVal, NOVALUE);
		}
	}
	return meter_data->returnVal;
}

SAMURIZE_PLUGIN_CHAR GetFAN2Speed(int id, char *updateIntervalString)
{	
	int updateInterval;
	i8k_meter *meter_data;

	// make sure driver loaded ok
	if(!isDriverLoaded) return NODRIVER;

	meter_data = (i8k_meter *)id;
	updateInterval = atoi(updateIntervalString);

	// see if data needs refresh
	if(NeedsRefresh(meter_data, updateInterval))
	{
		// try to retrieve data
		if(!GetFanSpeed(meter_data, FAN2ID)) strcpy(meter_data->returnVal, NOVALUE);
		else
		switch(meter_data->deviceVal)
		{
			case 0:
				strcpy( meter_data->returnVal, "Off");
				break;
			case 1:
				strcpy( meter_data->returnVal, "Slow");
				break;
			case 2:
				strcpy( meter_data->returnVal, "High");
				break;
			default:
				strcpy( meter_data->returnVal, NOVALUE);
		}

	}

	return meter_data->returnVal;
}


